@@ -8,9 +8,9 @@
  * of run-time information about current kernel
  */
 
-int kerndat_init(void);
-int kerndat_init_rst(void);
-int kerndat_get_dirty_track(void);
+extern int kerndat_init(void);
+extern int kerndat_init_rst(void);
+extern int kerndat_get_dirty_track(void);
 
 extern dev_t kerndat_shmem_dev;
 extern bool kerndat_has_dirty_track;
@@ -19,4 +19,5 @@ extern int tcp_max_wshare;
 extern int tcp_max_rshare;
 
 extern int kern_last_cap;
-#endif
+
+#endif /* __CR_KERNDAT_H__ */
