@@ -5,57 +5,62 @@
 #include <google/protobuf-c/protobuf-c.h>
 
 enum {
+	/* PB_AUTOGEN_START */
 	PB_INVENTORY,
 	PB_STATS,
 	PB_FDINFO,
 	PB_CORE,
 	PB_MM,
-	PB_VMAS,
-	PB_SIGACT,
-	PB_ITIMERS,
-	PB_POSIX_TIMERS,
+	PB_VMA,
+	PB_ITIMER,
+	PB_POSIX_TIMER,
 	PB_CREDS,
 	PB_FS,
 	PB_UTSNS,
-	PB_IPCNS_VAR,
-	PB_IPCNS_SHM,
-	PB_IPCNS_MSG,
-	PB_IPCNS_MSG_ENT,
-	PB_IPCNS_SEM,
-	PB_MOUNTPOINTS,
-	PB_NETDEV,
+	PB_IPC_VAR,
+	PB_IPC_SHM,
+	PB_IPC_SEM,
+	PB_MNT,
 	PB_PSTREE,
 	PB_GHOST_FILE,
 	PB_TCP_STREAM,
-	PB_SK_QUEUES,
-	PB_REG_FILES,
-	PB_NS_FILES,
-	PB_INETSK,
-	PB_UNIXSK,
-	PB_PACKETSK,
-	PB_NETLINKSK,
-	PB_PIPES,
+	PB_REG_FILE,
+	PB_NS_FILE,
+	PB_INET_SK,
+	PB_UNIX_SK,
+	PB_PACKET_SOCK,
+	PB_NETLINK_SK,
+	PB_PIPE,
 	PB_FIFO,
-	PB_PIPES_DATA,
-	PB_REMAP_FPATH,
-	PB_EVENTFD,
-	PB_EVENTPOLL,
+	PB_PIPE_DATA,
+	PB_EVENTFD_FILE,
+	PB_EVENTPOLL_FILE,
 	PB_EVENTPOLL_TFD,
 	PB_SIGNALFD,
-	PB_INOTIFY,
+	PB_INOTIFY_FILE,
 	PB_INOTIFY_WD,
-	PB_FANOTIFY,
+	PB_FANOTIFY_FILE,
 	PB_FANOTIFY_MARK,
-	PB_TTY,
+	PB_TTY_FILE,
 	PB_TTY_INFO,
 	PB_FILE_LOCK,
 	PB_RLIMIT,
-	PB_IDS,
-	PB_PAGEMAP_HEAD,
 	PB_PAGEMAP,
 	PB_SIGINFO,
+	PB_TUNFILE,
 
-	PB_MAX
+	/* PB_AUTOGEN_STOP */
+
+	PB_PAGEMAP_HEAD,
+	PB_IDS,
+	PB_SIGACT,
+	PB_NETDEV,
+	PB_REMAP_FPATH,
+	PB_SK_QUEUES,
+	PB_IPCNS_MSG,
+	PB_IPCNS_MSG_ENT,
+
+	PB_MAX,
 };
 
 typedef size_t (*pb_getpksize_t)(void *obj);
