@@ -11,7 +11,8 @@
 
 #include "asm/types.h"
 #include "libnetlink.h"
-#include "crtools.h"
+#include "cr_options.h"
+#include "fdset.h"
 #include "inet_diag.h"
 #include "files.h"
 #include "image.h"
