@@ -1,8 +1,7 @@
 #!/bin/bash
 
 source ../../../functions.sh || exit 1
-
-criu="../../../../criu"
+source ../../../env.sh || exit 1
 
 cleanup_class() {
 	rm -f ./*.class
